package core;

import exceptions.IllegalPositionException;
import interfaces.Tile;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Set;
import java.util.TreeSet;

/**
 *
 * @author reidl
 */
public class StandardTile implements Tile {

    private final int id;
    private TileType type;
    private Set<Integer> objectives = new TreeSet<Integer>();
    private Set<Integer> players = new TreeSet<Integer>();
    private boolean isMovable = true;

    protected StandardTile(int id, TileType type) {
        this.id = id;
	this.type = type;
    }

    @Override
    public TileType getType() {
	return type;
    }

    protected void setType(TileType type) {
	this.type = type;
    }
    
    public int getId() {
        return id;
    }

    /**
     * @return a list of possible types which are obtainable via rotation
     */
    public ArrayList<TileType> rotate() {
	ArrayList<TileType> res = new ArrayList<TileType>();
	// Implement me!
	return res;
    }

    @Override
    public Set<Integer> getPlayers() {
	return new TreeSet<Integer>(this.players);
    }

    @Override
    public Set<Integer> getObjectives() {
	return new TreeSet<Integer>(this.objectives);
    }

    @Override
    public void addPlayer(int id) {
        if( type == TileType.BLANK )
            throw new IllegalPositionException("A player was added to a "
                    + "position that contains a BLANK tile.");
	this.players.add(id);
    }

    @Override
    public void addObjective(int id) {
        if( type == TileType.BLANK )
            throw new IllegalPositionException("An objective was added to a "
                    + "position that contains a BLANK tile.");
	this.objectives.add(id);
    }
    
    @Override
    public void removeObjective(int id) {
        this.objectives.remove(id);
    }

    @Override
    public void setMovable(boolean isMovable) {
	this.isMovable = isMovable;
    }

    @Override
    public boolean isMovable() {
	return this.isMovable;
    }

    public void addPlayers(Collection<Integer> ids) {
        players.addAll(ids);
    }

    @Override
    public void removePlayer(int id) {
        this.players.remove(id);
    }
    
    @Override
    public Set<Integer> removePlayers() {
        Set<Integer> res = new TreeSet<Integer>(players);
        players.clear();
        return res;
    }

    public StandardTile copy() {
        StandardTile res = new StandardTile(id, type);
        res.isMovable = isMovable;
        res.objectives.addAll(objectives);
        res.players.addAll(players);
        return res;
    }

    @Override
    public int hashCode() {
        int hash = 3;
        hash = 13 * hash + this.id;
        hash = 13 * hash + (this.type != null ? this.type.hashCode() : 0);
        hash = 13 * hash + (this.objectives != null ? this.objectives.hashCode() : 0);
        hash = 13 * hash + (this.players != null ? this.players.hashCode() : 0);
        hash = 13 * hash + (this.isMovable ? 1 : 0);
        return hash;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final StandardTile other = (StandardTile) obj;
        if (this.id != other.id) {
            return false;
        }
        if (this.type != other.type) {
            return false;
        }
        if (this.objectives != other.objectives && (this.objectives == null || !this.objectives.equals(other.objectives))) {
            return false;
        }
        if (this.players != other.players && (this.players == null || !this.players.equals(other.players))) {
            return false;
        }
        if (this.isMovable != other.isMovable) {
            return false;
        }
        return true;
    }
}
