package tools;

import java.util.Iterator;

/**
 *
 * @author ubik
 */
public class Iterators {
    public static <T> Iterable<T> join(Iterable<? extends T> itrbA, Iterable<? extends T> itrbB) {
        final Iterator<? extends T> itA = itrbA.iterator();
        final Iterator<? extends T> itB = itrbB.iterator();
        return wrap(new Iterator<T>() {
            @Override
            public boolean hasNext() {
                if( itA.hasNext() ) {
                    return true;
                }
                return itB.hasNext();
            }

            @Override
            public T next() {
                if( itA.hasNext() ) {
                    return itA.next();
                }
                return itB.next();
            }

            @Override
            public void remove() {
                throw new UnsupportedOperationException("Not supported.");
            }
        });
    }
    
    public static <T> Iterable<T> wrap(final Iterator<T> it) {
        return new Iterable<T>() {
            @Override
            public Iterator<T> iterator() {
                return it;
            }
        };
    }

    private static class IterableWrap<T> implements Iterable<T> {
        private Iterator<T> it;
        private IterableWrap(Iterator<T> it) {
            this.it = it;
        }
        
        @Override
        public Iterator<T> iterator() {
            return it;
        }
    }
}
